#include "IP.h"

extern int verbose;

void doIP(Graph& G, Sol& C, double timeLimit, bool doingSCycles) {
	int u, v, l, i;
	double f, overallRunTime;
	string command, str;

	//Write this graph to a file in the format for the IP program (FICO Xpress here). If we're doing s-cycles, the vertices are relabelled so that the source = 0
	vector<int> map(G.n);
	for (i = 0; i < G.n; i++) map[i] = i;
	if (doingSCycles) swapVals(map[0], map[G.source]);
	ofstream fout("inx.txt");
	fout << "c Temporary file for the IP solver\n";
	fout << "p " << G.n << " " << G.m << "\n";
	for (i = 0; i < G.n; i++) {
		fout << "v " << i << "\n";
	}
	for (u = 0; u < G.n; u++) {
		for (Neighbour el : G.AList[u]) {
			if (u < el.vertex) {
				fout << "e " << map[u] << " " << map[el.vertex] << " " << el.weight << "\n";
			}
		}
	}
	fout.close();

	if (doingSCycles) {
		if (verbose >= 1) cout << "Executing IP algorithm in <k-s-cycle.mos> ...\n";
		command = "mosel k-s-cycle.mos k=" + to_string(G.k) + " fname=inx.txt timelimit=" + to_string(int(timeLimit)) + " verbose=" + to_string(verbose) + " outfile=outx.txt";
		if (verbose >= 1) cout << command << endl;
		system(command.c_str());
	}
	else {
		if (verbose >= 1) cout << "Executing IP algorithm in <k-cycle.mos> ...\n";
		command = "mosel k-cycle.mos k=" + to_string(G.k) + " fname=inx.txt timelimit=" + to_string(int(timeLimit)) + " verbose=" + to_string(verbose) + " outfile=outx.txt";
		if (verbose >= 1) cout << command << endl;
		system(command.c_str());
	}

	//Get info from the out file from the IP solver. This is put into CNew. If the file contains "NULL" then no integer solution was found, else we assume the
	//file is valid. A valid outfile contains the following information:
	//overallRunTime, optimumFound(true,false), totalEdgeWeightOfSolution, numVerticesInSolution
	//This is followed by the solution itself
	ifstream fin("outx.txt");
	fin >> str;
	if (str == "NULL" || str == "Null" || str == "null") {
		//This check is needed in case IP did not find any integer solution within the time limit.
		if (verbose >= 1) cout << "No integer solution returned by IP.\n";
		return;
	}
	else {
		overallRunTime = stod(str);
	}
	fin >> str >> f >> l;
	Sol CNew;
	for (i = 0; i < l; i++) {
		fin >> u;
		CNew.seq.push_back(map[u]);
	}
	fin.close();
	CNew.len = int(f + 0.5);
	if (str == "true") CNew.isGlobalOptimum = true;

	//Delete the temporary files that were created
	command = "del inx.txt";
	system(command.c_str());
	command = "del outx.txt";
	system(command.c_str());

	//Finally, replace C with the CNew if it is better
	if (abs(CNew.len - G.k) < abs(C.len - G.k)) {
		C = CNew;
		C.improvedByOptimiser = true;
	}
	else if (abs(CNew.len - G.k) == abs(C.len - G.k) && CNew.isGlobalOptimum) {
		C.isGlobalOptimum = true;
	}
}