#include "Yen.h"

extern int verbose;

void addDummyVetex(Graph& G) {
	//Adds a new vertex sPrime to the graph. All of s's in-edges are then redirected to sPrime
	int s = G.source, sPrime = G.n, v, w;
	G.AList.push_back(set<Neighbour, minNeighbour>());
	G.n++;
	for (Neighbour el : G.AList[s]) {
		v = el.vertex;
		w = el.weight;
		//Remove edge (v,s) and add edge (v,sPrime,w)
		deleteArc(G, v, s);
		addArc(G, v, sPrime, w);
	}
}
void removeDummyVertex(Graph& G) {
	//Redirects all of SPrime's in-edges to s, then removes sPrime all together
	G.n--;
	G.AList.pop_back();
	int sPrime = G.n, s = G.source, v, w;
	for (Neighbour el : G.AList[s]) {
		v = el.vertex;
		w = el.weight;
		//Remove edge (v,sPrime) and add edge (v,s,w)
		deleteArc(G, v, sPrime);
		addArc(G, v, s, w);
	}
}
void doYen(Graph& G, Sol& C, double timeLimit) {
	int u, l, i, numIts;
	double f, overallRunTime;
	string command, str;

	//First modify the graph by making a dummy vertex sPrime that takes all the in-edges of s
	addDummyVetex(G);

	//Write this graph to a file in the format for Yen's algorithm program
	ofstream fout("inx.txt");
	for (u = 0; u < G.n; u++) for (Neighbour el : G.AList[u]) fout << u << " " << el.vertex << " " << el.weight << "\n";
	fout.close();

	//And reset the graph by deleting sPrime and re-directing its in-edges back to s
	removeDummyVertex(G);

	//Execute Yen's algorithm
	if (verbose >= 1) cout << "Executing <Yen.jar> ...\n";
	command = "java -jar Yen.jar -i inx.txt -K -1 -T " + to_string(G.k) + " -TL " + to_string(timeLimit) + " -s " + to_string(G.source) + " -t " + to_string(G.n);
	for (i = 0; i < verbose; i++) command += " -v";
	system(command.c_str());

	//Get info from the out file produced by Yen's algorithm. Remeber to ignore the last vertex sPrime and to add s instead, this is put into the solution CYen.
	//The outfile contains the following information:
	//overallRunTime, numItsOfYen, optimumFound(true,false), totalEdgeWeightOfSolution, numVerticesInSolution
	//This is followed by the solution itself
	ifstream fin("outx.txt");
	fin >> str;
	if (str == "NULL" || str == "Null" || str == "null") {
		//This check is needed in case Yen was not able to find any valid solution within the time limit.
		if (verbose >= 1) cout << "No solution found by Yen's algorithm.\n";
		return;
	}
	else {
		overallRunTime = stod(str);
	}
	fin >> numIts >> str >> f >> l;
	Sol CYen;
	for (i = 0; i < l - 1; i++) {
		fin >> u;
		CYen.seq.push_back(u);
	}
	CYen.seq.push_back(G.source);
	fin.close();
	CYen.len = int(f + 0.5);
	if (str == "true") CYen.isGlobalOptimum = true;

	//Delete the temporary files that were created
	command = "del inx.txt";
	system(command.c_str());
	command = "del outx.txt";
	system(command.c_str());

	//The following check is necessary, because sometimes the returned "cycle" will only have two edges (s,v) and (v,s), which is wrong.
	if (CYen.seq.size() <= 2) {
		logAndExit("The solution returned by Yen's algorithm contains only " + to_string(C.seq.size()) + " edges, which is invalid.\n");
	}

	//Finally, replace C with the solution from Yen's algorithm if it is better
	if (abs(CYen.len - G.k) < abs(C.len - G.k)) {
		C = CYen;
		C.improvedByOptimiser = true;
	}
	else if (abs(CYen.len - G.k) == abs(C.len - G.k) && CYen.isGlobalOptimum) {
		C.isGlobalOptimum = true;
	}
}