#ifndef MAIN
#define MAIN

#include <time.h>
#include <climits>
#include <string>
#include <fstream>
#include <iostream>
#include <iomanip>
#include <vector>
#include <set>
#include <list>
#include <deque>
#include <map>
#include <stack>
#include <algorithm>
#include <math.h>

using namespace std;
#pragma warning(disable: 4267)

void logAndExit(string s);

//Global constants
const int infty = INT_MAX;
const int null = -1;
const int white = 0;
const int grey = 1;
const int black = 2;

//Structs used in conjuntion with the priority queue
struct LabelItem {
	int vertex;
	int label;
};
struct minLabel {
	bool operator() (const LabelItem& lhs, const LabelItem& rhs) const {
		return tie(lhs.label, lhs.vertex) < tie(rhs.label, rhs.vertex);
	}
};
struct minVertex {
	bool operator() (const LabelItem& lhs, const LabelItem& rhs) const {
		return tie(lhs.vertex, lhs.label) < tie(rhs.vertex, rhs.label);
	}
};

//Structs for holding the neighbours of a vertex. This is used as part of an adjacency list representation
struct Neighbour {
	int vertex;
	int weight;
};
struct minNeighbour {
	bool operator() (const Neighbour& lhs, const Neighbour& rhs) const {
		return lhs.vertex < rhs.vertex;
	}
};

//Structs for holding a weighted arc (ordered pair)
struct WArc {
	int u;
	int v;
	int weight;
};
struct minWArc {
	bool operator() (const WArc& lhs, const WArc& rhs) const {
		return tie(lhs.u, lhs.v, lhs.weight) < tie(rhs.u, rhs.v, rhs.weight);
	}
};
struct minWArcByWeight {
	bool operator() (const WArc& lhs, const WArc& rhs) const {
		return tie(lhs.weight, lhs.u, lhs.v) < tie(rhs.weight, rhs.u, rhs.v);
	}
};
struct maxWArcByWeight {
	bool operator() (const WArc& lhs, const WArc& rhs) const {
		return tie(lhs.weight, lhs.u, lhs.v) > tie(rhs.weight, rhs.u, rhs.v);
	}
};

//Struct for holding a solution
struct Sol {
	int len = 0;
	vector<int> seq = {};
	vector<int> cumulative = {};
	bool atLocalOpt = false;
	bool isGlobalOptimum = false;
	bool improvedByOptimiser = false;
};

//Struct used for storing a graph
struct Graph {
	int n = 0;										//Number of vertices in the graph
	int m = 0;										//Number of edges in a graph.
	int source = -1;								//The source vertex for this graph (<0 means there is no source)
	int k = 0;										//The desired length of the cycle
	vector<set<Neighbour, minNeighbour> > AList;	//Adjacency list (each list is a set ordered by vetex)
	int LenUB = 0;									//Upper bound on the length of a cycle (sum of the n heaviest edges)
	vector<string> vertexID;						//Actual name of the vertex
	vector<vector<Neighbour> > AVec;				//Version of the adjacency list structure used with random path fining algorithms
	bool operator<(const Graph& rhs) const			//Comparison operator for comparing two graph sizes (n+m)
	{
		return((m + n) < (rhs.m + rhs.n));
	}
};

struct GraphStats {
	int n = 0;										//Number of vertices in the graph
	int m = 0;										//Number of edges in a graph.
	int k = 0;										//Desired length
	int source = -1;								//By default no source is assumed
	int minWeight = infty;
	int maxWeight = 0;								//Keeps track of the largest edge weight in the graph
	int totalWeight = 0;							//Sum of all the edge weights
	double meanWeight = 0.0;
	int minWeightedDeg = infty;
	int minDeg = INT_MAX;
	int maxWeightedDeg = 0;
	int maxDeg = 0;
	Sol shortestCycle = { 0, {}, {}, false };
	int LB1 = 0;
	int LB2 = 0;
	int LB3 = 0;
	int LB4 = 0;
	int UB1 = 0;
	int UB2 = 0;
	int UB3 = 0;
};

int minVal(int a, int b);
void logAndExit(string s);
void swapVals(int& a, int& b);
double stDev(vector<int>& X);

#endif //MAIN